@@ -69,12 +69,61 @@ struct _MetaCompositorVulkan
 
   uint32_t                  n_framebuffers;
   VkFramebuffer            *framebuffers;
+
+  uint32_t                  n_command_buffers;
+  VkCommandBuffer          *command_buffers;
 #endif
 };
 
 G_DEFINE_TYPE (MetaCompositorVulkan, meta_compositor_vulkan, META_TYPE_COMPOSITOR)
 
 #ifdef HAVE_VULKAN
+static void
+destroy_command_buffers (MetaCompositorVulkan *vulkan)
+{
+  if (vulkan->command_buffers == NULL)
+    return;
+
+  vkFreeCommandBuffers (vulkan->device, vulkan->command_pool,
+                        vulkan->n_command_buffers, vulkan->command_buffers);
+
+  g_clear_pointer (&vulkan->command_buffers, g_free);
+
+  vulkan->n_command_buffers = 0;
+}
+
+static gboolean
+create_command_buffers (MetaCompositorVulkan  *vulkan,
+                        GError               **error)
+{
+  VkCommandBufferAllocateInfo allocate_info;
+  VkResult result;
+
+  destroy_command_buffers (vulkan);
+
+  vulkan->n_command_buffers = vulkan->n_images;
+  vulkan->command_buffers = g_new0 (VkCommandBuffer, vulkan->n_images);
+
+  allocate_info.sType = VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO;
+  allocate_info.pNext = NULL;
+  allocate_info.commandPool = vulkan->command_pool;
+  allocate_info.level = VK_COMMAND_BUFFER_LEVEL_PRIMARY;
+  allocate_info.commandBufferCount = vulkan->n_command_buffers;
+
+  result = vkAllocateCommandBuffers (vulkan->device, &allocate_info,
+                                     vulkan->command_buffers);
+
+  if (result != VK_SUCCESS)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Failed to allocate command buffers");
+
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static void
 destroy_framebuffers (MetaCompositorVulkan *vulkan)
 {
@@ -1085,6 +1134,7 @@ meta_compositor_vulkan_finalize (GObject *object)
 
   vulkan = META_COMPOSITOR_VULKAN (object);
 
+  destroy_command_buffers (vulkan);
   destroy_framebuffers (vulkan);
   destroy_render_pass (vulkan);
   destroy_image_views (vulkan);
@@ -1208,6 +1258,9 @@ meta_compositor_vulkan_manage (MetaCompositor  *compositor,
   if (!create_framebuffers (vulkan, error))
     return FALSE;
 
+  if (!create_command_buffers (vulkan, error))
+    return FALSE;
+
   g_timeout_add (10000, (GSourceFunc) not_implemented_cb, vulkan);
 
   return TRUE;
@@ -1252,8 +1305,14 @@ meta_compositor_vulkan_window_opacity_changed (MetaCompositor *compositor,
 }
 
 static void
-meta_compositor_vulkan_window_shape_changed (MetaCompositor *compositor,
-                                             MetaWindow     *window)
+meta_compositor_vulkan_window_opaque_region_changed (MetaCompositor *compositor,
+                                                     MetaWindow     *window)
+{
+}
+
+static void
+meta_compositor_vulkan_window_shape_region_changed (MetaCompositor *compositor,
+                                                    MetaWindow     *window)
 {
 }
 
@@ -1329,7 +1388,8 @@ meta_compositor_vulkan_class_init (MetaCompositorVulkanClass *vulkan_class)
   compositor_class->show_window = meta_compositor_vulkan_show_window;
   compositor_class->hide_window = meta_compositor_vulkan_hide_window;
   compositor_class->window_opacity_changed = meta_compositor_vulkan_window_opacity_changed;
-  compositor_class->window_shape_changed = meta_compositor_vulkan_window_shape_changed;
+  compositor_class->window_opaque_region_changed = meta_compositor_vulkan_window_opaque_region_changed;
+  compositor_class->window_shape_region_changed = meta_compositor_vulkan_window_shape_region_changed;
   compositor_class->set_updates_frozen = meta_compositor_vulkan_set_updates_frozen;
   compositor_class->process_event = meta_compositor_vulkan_process_event;
   compositor_class->get_window_surface = meta_compositor_vulkan_get_window_surface;
