@@ -627,9 +627,6 @@ shadow_picture_clip (Display          *xdisplay,
   XserverRegion region1;
   XserverRegion region2;
 
-  if (!cw->window)
-    return;
-
   visible_region = meta_window_get_frame_bounds (cw->window);
 
   if (!visible_region)
@@ -973,38 +970,31 @@ window_has_shadow (MetaCompositorXRender *xrender,
   if (xrender->have_shadows == FALSE)
     return FALSE;
 
-  /* Always put a shadow around windows with a frame. This should override
-   * the restriction about not putting a shadow around shaped windows as the
-   * frame might be the reason the window is shaped.
-   */
-  if (cw->window)
+  /* Do not add shadows to fullscreen windows */
+  if (meta_window_is_fullscreen (cw->window))
     {
-      /* Do not add shadows to fullscreen windows */
-      if (meta_window_is_fullscreen (cw->window))
-        {
-          meta_verbose ("Window has no shadow because it is fullscreen\n");
-          return FALSE;
-        }
+      meta_verbose ("Window has no shadow because it is fullscreen\n");
+      return FALSE;
+    }
 
-      /* Do not add shadows to maximized windows */
-      if (meta_window_is_maximized (cw->window))
-        {
-          meta_verbose ("Window has no shadow because it is maximized\n");
-          return FALSE;
-        }
+  /* Do not add shadows to maximized windows */
+  if (meta_window_is_maximized (cw->window))
+    {
+      meta_verbose ("Window has no shadow because it is maximized\n");
+      return FALSE;
+    }
 
-      /* Do not add shadows if GTK+ theme is used */
-      if (meta_prefs_get_theme_type () == META_THEME_TYPE_GTK)
-        {
-          meta_verbose ("Window has shadow from GTK+ theme\n");
-          return FALSE;
-        }
+  /* Do not add shadows if GTK+ theme is used */
+  if (meta_prefs_get_theme_type () == META_THEME_TYPE_GTK)
+    {
+      meta_verbose ("Window has shadow from GTK+ theme\n");
+      return FALSE;
+    }
 
-      if (meta_window_get_frame (cw->window))
-        {
-          meta_verbose ("Window has shadow because it has a frame\n");
-          return TRUE;
-        }
+  if (meta_window_get_frame (cw->window))
+    {
+      meta_verbose ("Window has shadow because it has a frame\n");
+      return TRUE;
     }
 
   /* Do not add shadows to ARGB windows */
@@ -1062,18 +1052,12 @@ win_extents (MetaCompositorXRender *xrender,
 
   if (cw->needs_shadow)
     {
+      MetaFrame *frame;
       MetaFrameBorders borders;
       XRectangle sr;
 
-      meta_frame_borders_clear (&borders);
-
-      if (cw->window)
-        {
-          MetaFrame *frame = meta_window_get_frame (cw->window);
-
-          if (frame)
-            meta_frame_calc_borders (frame, &borders);
-        }
+      frame = meta_window_get_frame (cw->window);
+      meta_frame_calc_borders (frame, &borders);
 
       cw->shadow_dx = (int) shadow_offsets_x [cw->shadow_type] + borders.invisible.left;
       cw->shadow_dy = (int) shadow_offsets_y [cw->shadow_type] + borders.invisible.top;
@@ -1165,7 +1149,7 @@ get_client_region (MetaDisplay    *display,
         return None;
     }
 
-  frame = cw->window ? meta_window_get_frame (cw->window) : NULL;
+  frame = meta_window_get_frame (cw->window);
 
   if (frame != NULL)
     {
@@ -1204,6 +1188,8 @@ get_visible_region (MetaDisplay    *display,
 {
   Display *xdisplay;
   XserverRegion region;
+  cairo_region_t *visible;
+  XserverRegion tmp;
 
   xdisplay = meta_display_get_xdisplay (display);
 
@@ -1219,20 +1205,14 @@ get_visible_region (MetaDisplay    *display,
         return None;
     }
 
-  if (cw->window)
+  visible = meta_window_get_frame_bounds (cw->window);
+  tmp = cairo_region_to_xserver_region (xdisplay, visible);
+
+  if (tmp != None)
     {
-      cairo_region_t *visible;
-      XserverRegion tmp;
-
-      visible = meta_window_get_frame_bounds (cw->window);
-      tmp = cairo_region_to_xserver_region (xdisplay, visible);
-
-      if (tmp != None)
-        {
-          XFixesTranslateRegion (xdisplay, tmp, cw->rect.x, cw->rect.y);
-          XFixesIntersectRegion (xdisplay, region, region, tmp);
-          XFixesDestroyRegion (xdisplay, tmp);
-        }
+      XFixesTranslateRegion (xdisplay, tmp, cw->rect.x, cw->rect.y);
+      XFixesIntersectRegion (xdisplay, region, region, tmp);
+      XFixesDestroyRegion (xdisplay, tmp);
     }
 
   return region;
@@ -1310,9 +1290,6 @@ get_window_mask (MetaDisplay    *display,
   cairo_t *cr;
   Picture picture;
 
-  if (cw->window == NULL)
-    return None;
-
   frame = meta_window_get_frame (cw->window);
   if (frame == NULL)
     return None;
@@ -1523,7 +1500,7 @@ paint_windows (MetaCompositorXRender *xrender,
           wid = cw->rect.width;
           hei = cw->rect.height;
 
-          frame = cw->window ? meta_window_get_frame (cw->window) : NULL;
+          frame = meta_window_get_frame (cw->window);
           meta_frame_calc_borders (frame, &borders);
 
           XFixesSetPictureClipRegion (xdisplay, root_buffer,
@@ -2303,7 +2280,7 @@ update_shadows (MetaPreference pref,
 
       cw = (MetaCompWindow *) index->data;
 
-      if (cw->window && cw->shadow)
+      if (cw->shadow != None)
         {
           XRenderFreePicture (xrender->xdisplay, cw->shadow);
           cw->shadow = None;
@@ -2470,6 +2447,8 @@ meta_compositor_xrender_add_window (MetaCompositor *compositor,
   MetaCompWindow *cw;
   Window xwindow;
 
+  g_assert (window != NULL);
+
   xrender = META_COMPOSITOR_XRENDER (compositor);
   display = meta_compositor_get_display (compositor);
 
@@ -2500,6 +2479,12 @@ meta_compositor_xrender_add_window (MetaCompositor *compositor,
   cw->shape_region = cairo_region_to_xserver_region (xrender->xdisplay,
                                                      window->shape_region);
 
+  if (cw->shape_region != None)
+    {
+      XFixesTranslateRegion (xrender->xdisplay, cw->shape_region,
+                             cw->rect.x, cw->rect.y);
+    }
+
   xwindow = get_toplevel_xwindow (window);
   cw->damage = XDamageCreate (xrender->xdisplay, xwindow, XDamageReportNonEmpty);
 
@@ -2516,7 +2501,7 @@ meta_compositor_xrender_add_window (MetaCompositor *compositor,
   cw->shadow_width = 0;
   cw->shadow_height = 0;
 
-  if (window && meta_window_has_focus (window))
+  if (meta_window_has_focus (window))
     cw->shadow_type = META_SHADOW_LARGE;
   else
     cw->shadow_type = META_SHADOW_MEDIUM;
@@ -2650,8 +2635,14 @@ meta_compositor_xrender_window_opacity_changed (MetaCompositor *compositor,
 }
 
 static void
-meta_compositor_xrender_window_shape_changed (MetaCompositor *compositor,
-                                              MetaWindow     *window)
+meta_compositor_xrender_window_opaque_region_changed (MetaCompositor *compositor,
+                                                      MetaWindow     *window)
+{
+}
+
+static void
+meta_compositor_xrender_window_shape_region_changed (MetaCompositor *compositor,
+                                                     MetaWindow     *window)
 {
   MetaCompositorXRender *xrender;
   MetaCompWindow *cw;
@@ -2664,9 +2655,6 @@ meta_compositor_xrender_window_shape_changed (MetaCompositor *compositor,
 
   if (cw->shape_region != None)
     {
-      XFixesTranslateRegion (xrender->xdisplay, cw->shape_region,
-                             cw->rect.x, cw->rect.y);
-
       dump_xserver_region (xrender, "shape_changed", cw->shape_region);
       add_damage (xrender, cw->shape_region);
 
@@ -2675,6 +2663,12 @@ meta_compositor_xrender_window_shape_changed (MetaCompositor *compositor,
 
   cw->shape_region = cairo_region_to_xserver_region (xrender->xdisplay,
                                                      window->shape_region);
+
+  if (cw->shape_region != None)
+    {
+      XFixesTranslateRegion (xrender->xdisplay, cw->shape_region,
+                             cw->rect.x, cw->rect.y);
+    }
 }
 
 static void
@@ -2914,8 +2908,8 @@ meta_compositor_xrender_sync_stack (MetaCompositor *compositor,
     }
 
   xrender->windows = g_list_reverse (xrender->windows);
-  xrender->clip_changed = TRUE;
 
+  damage_screen (xrender);
   add_repair (xrender);
 }
 
@@ -2925,7 +2919,7 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
 {
   MetaCompositorXRender *xrender;
   MetaCompWindow *cw;
-  MetaRectangle rect;
+  MetaRectangle old_rect;
   XserverRegion damage;
 
   xrender = META_COMPOSITOR_XRENDER (compositor);
@@ -2936,7 +2930,8 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
 
   meta_error_trap_push (window->display);
 
-  meta_window_get_input_rect (window, &rect);
+  old_rect = cw->rect;
+  meta_window_get_input_rect (window, &cw->rect);
 
   if (xrender->debug)
     {
@@ -2944,7 +2939,7 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
                cw->shape_region != None, cw->needs_shadow);
       dump_xserver_region (xrender, "\textents", cw->extents);
       fprintf (stderr, "\txy (%d %d), wh (%d %d)\n",
-               rect.x, rect.y, rect.width, rect.height);
+               cw->rect.x, cw->rect.y, cw->rect.width, cw->rect.height);
     }
 
   if (cw->extents)
@@ -2959,7 +2954,7 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
         fprintf (stderr, "no extents to damage !\n");
     }
 
-  if (cw->rect.width != rect.width || cw->rect.height != rect.height)
+  if (cw->rect.width != old_rect.width || cw->rect.height != old_rect.height)
     {
       if (cw->shaded.back_pixmap != None)
         {
@@ -2984,7 +2979,7 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
           /* If the window is shaded, we store the old backing pixmap
            * so we can return a proper image of the window
            */
-          if (cw->window && meta_window_is_shaded (cw->window))
+          if (meta_window_is_shaded (cw->window))
             {
               cw->shaded.back_pixmap = cw->back_pixmap;
               cw->back_pixmap = None;
@@ -3001,7 +2996,7 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
           /* If the window is shaded, we store the old backing pixmap
            * so we can return a proper image of the window
            */
-          if (cw->window && meta_window_is_shaded (cw->window))
+          if (meta_window_is_shaded (cw->window))
             {
               cw->shaded.mask_pixmap = cw->mask_pixmap;
               cw->mask_pixmap = None;
@@ -3013,12 +3008,12 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
             }
         }
 
-      if (cw->window && meta_window_is_shaded (cw->window))
+      if (meta_window_is_shaded (cw->window))
         {
-          cw->shaded.x = cw->rect.x;
-          cw->shaded.y = cw->rect.y;
-          cw->shaded.width = cw->rect.width;
-          cw->shaded.height = cw->rect.height;
+          cw->shaded.x = old_rect.x;
+          cw->shaded.y = old_rect.y;
+          cw->shaded.width = old_rect.width;
+          cw->shaded.height = old_rect.height;
 
           if (cw->client_region != None)
             {
@@ -3049,8 +3044,6 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
         }
     }
 
-  cw->rect = rect;
-
   if (cw->extents)
     XFixesDestroyRegion (xrender->xdisplay, cw->extents);
 
@@ -3069,6 +3062,18 @@ meta_compositor_xrender_sync_window_geometry (MetaCompositor *compositor,
       XFixesCopyRegion (xrender->xdisplay, damage, cw->extents);
     }
 
+  if (cw->shape_region != None)
+    {
+      gint dx;
+      gint dy;
+
+      dx = cw->rect.x - old_rect.x;
+      dy = cw->rect.y - old_rect.y;
+
+      XFixesUnionRegion (xrender->xdisplay, damage, damage, cw->shape_region);
+      XFixesTranslateRegion (xrender->xdisplay, cw->shape_region, dx, dy);
+    }
+
   dump_xserver_region (xrender, "sync_window_geometry", damage);
   add_damage (xrender, damage);
 
@@ -3117,7 +3122,8 @@ meta_compositor_xrender_class_init (MetaCompositorXRenderClass *xrender_class)
   compositor_class->show_window = meta_compositor_xrender_show_window;
   compositor_class->hide_window = meta_compositor_xrender_hide_window;
   compositor_class->window_opacity_changed = meta_compositor_xrender_window_opacity_changed;
-  compositor_class->window_shape_changed = meta_compositor_xrender_window_shape_changed;
+  compositor_class->window_opaque_region_changed = meta_compositor_xrender_window_opaque_region_changed;
+  compositor_class->window_shape_region_changed = meta_compositor_xrender_window_shape_region_changed;
   compositor_class->set_updates_frozen = meta_compositor_xrender_set_updates_frozen;
   compositor_class->process_event = meta_compositor_xrender_process_event;
   compositor_class->get_window_surface = meta_compositor_xrender_get_window_surface;
