@@ -774,8 +774,7 @@ cache_edges (MetaDisplay *display,
   /*
    * 0th: Print debugging information to the log about the edges
    */
-#ifdef WITH_VERBOSE_MODE
-  if (meta_is_verbose())
+  if (meta_check_debug_flags (META_DEBUG_EDGE_RESISTANCE))
     {
       int max_edges = MAX (MAX( g_list_length (window_edges),
                                 g_list_length (xinerama_edges)),
@@ -794,7 +793,6 @@ cache_edges (MetaDisplay *display,
       meta_topic (META_DEBUG_EDGE_RESISTANCE,
                   "Screen edges for resistance  : %s\n", big_buffer);
     }
-#endif
 
   /*
    * 1st: Get the total number of each kind of edge
