@@ -67,11 +67,6 @@
 #include <time.h>
 #include <unistd.h>
 
-/**
- * The exit code we'll return to our parent process when we eventually die.
- */
-static MetaExitCode meta_exit_code = META_EXIT_SUCCESS;
-
 /**
  * Handle on the main loop, so that we have an easy way of shutting Metacity
  * down.
@@ -84,9 +79,6 @@ static GMainLoop *meta_main_loop = NULL;
  */
 static gboolean meta_restart_after_quit = FALSE;
 
-static void prefs_changed_callback (MetaPreference pref,
-                                    gpointer       data);
-
 /**
  * Prints the version notice. This is shown when Metacity is called
  * with the --version switch.
@@ -117,8 +109,7 @@ version (void)
 /**
  * Prints a list of which configure script options were used to
  * build this copy of Metacity. This is actually always called
- * on startup, but it's all no-op unless we're in verbose mode
- * (see meta_set_verbose).
+ * on startup, but it's all no-op unless we're in verbose mode.
  */
 static void
 meta_print_compilation_info (void)
@@ -155,8 +146,7 @@ meta_print_compilation_info (void)
  * build date), the locale, the character encoding, and a list
  * of configure script options that were used to build this
  * copy of Metacity. This is actually always called
- * on startup, but it's all no-op unless we're in verbose mode
- * (see meta_set_verbose).
+ * on startup, but it's all no-op unless we're in verbose mode.
  */
 static void
 meta_print_self_identity (void)
@@ -344,7 +334,7 @@ sigterm_handler (int signum)
 static gboolean
 on_sigterm (void)
 {
-  meta_quit (META_EXIT_SUCCESS);
+  meta_quit ();
   return FALSE;
 }
 
@@ -368,7 +358,8 @@ main (int argc, char **argv)
   GIOChannel *channel;
 
   if (setlocale (LC_ALL, "") == NULL)
-    meta_warning ("Locale not understood by C library, internationalization will not work\n");
+    g_warning ("Locale not understood by C library, internationalization "
+               "will not work");
 
   sigemptyset (&empty_mask);
   act.sa_handler = SIG_IGN;
@@ -398,15 +389,16 @@ main (int argc, char **argv)
     g_printerr ("Failed to register SIGTERM handler: %s\n",
 		g_strerror (errno));
 
-  if (g_getenv ("METACITY_VERBOSE"))
-    meta_set_verbose (TRUE);
+  meta_init_debug ();
+
   if (g_getenv ("METACITY_DEBUG"))
     meta_set_debugging (TRUE);
 
   if (g_get_home_dir ())
-    if (chdir (g_get_home_dir ()) < 0)
-      meta_warning ("Could not change to home directory %s.\n",
-                    g_get_home_dir ());
+    {
+      if (chdir (g_get_home_dir ()) < 0)
+        g_warning ("Could not change to home directory %s.", g_get_home_dir ());
+    }
 
   meta_print_self_identity ();
 
@@ -428,7 +420,10 @@ main (int argc, char **argv)
     meta_set_replace_current_wm (TRUE);
 
   if (meta_args.save_file && meta_args.client_id)
-    meta_fatal ("Can't specify both SM save file and SM client id\n");
+    {
+      g_critical ("Can't specify both SM save file and SM client id");
+      exit (EXIT_FAILURE);
+    }
 
   meta_main_loop = g_main_loop_new (NULL, FALSE);
 
@@ -436,9 +431,6 @@ main (int argc, char **argv)
 
   /* Load prefs */
   meta_prefs_init ();
-  meta_prefs_add_listener (prefs_changed_callback, NULL);
-
-  meta_ui_reload_theme ();
 
   /* Connect to SM as late as possible - but before managing display,
    * or we might try to manage a window before we have the session
@@ -476,7 +468,9 @@ main (int argc, char **argv)
     meta_prefs_set_force_fullscreen (FALSE);
 
   if (!meta_display_open ())
-    meta_exit (META_EXIT_ERROR);
+    {
+      exit (EXIT_FAILURE);
+    }
 
   g_main_loop_run (meta_main_loop);
 
@@ -496,14 +490,14 @@ main (int argc, char **argv)
                           NULL,
                           &err))
         {
-          meta_fatal (_("Failed to restart: %s\n"),
-                      err->message);
-          g_error_free (err); /* not reached anyhow */
-          meta_exit_code = META_EXIT_ERROR;
+          g_critical ("Failed to restart: %s", err->message);
+          g_error_free (err);
+
+          return EXIT_FAILURE;
         }
     }
 
-  return meta_exit_code;
+  return EXIT_SUCCESS;
 }
 
 /**
@@ -516,10 +510,8 @@ main (int argc, char **argv)
  * \param code The success or failure code to return to the calling process.
  */
 void
-meta_quit (MetaExitCode code)
+meta_quit (void)
 {
-  meta_exit_code = code;
-
   if (g_main_loop_is_running (meta_main_loop))
     g_main_loop_quit (meta_main_loop);
 }
@@ -534,68 +526,5 @@ void
 meta_restart (void)
 {
   meta_restart_after_quit = TRUE;
-  meta_quit (META_EXIT_SUCCESS);
-}
-
-/**
- * Called on pref changes. (One of several functions of its kind and purpose.)
- *
- * \bug Why are these particular prefs handled in main.c and not others?
- * Should they be?
- *
- * \param pref  Which preference has changed
- * \param data  Arbitrary data (which we ignore)
- */
-static void
-prefs_changed_callback (MetaPreference pref,
-                        gpointer       data)
-{
-  switch (pref)
-    {
-    case META_PREF_THEME_NAME:
-    case META_PREF_THEME_TYPE:
-    case META_PREF_COMPOSITING_MANAGER:
-      meta_ui_reload_theme ();
-      meta_display_retheme_all ();
-      break;
-
-    case META_PREF_CURSOR_THEME:
-    case META_PREF_CURSOR_SIZE:
-      meta_display_set_cursor_theme (meta_prefs_get_cursor_theme (),
-				     meta_prefs_get_cursor_size ());
-      break;
-
-    case META_PREF_MOUSE_BUTTON_MODS:
-    case META_PREF_FOCUS_MODE:
-    case META_PREF_FOCUS_NEW_WINDOWS:
-    case META_PREF_ATTACH_MODAL_DIALOGS:
-    case META_PREF_RAISE_ON_CLICK:
-    case META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR:
-    case META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR:
-    case META_PREF_ACTION_RIGHT_CLICK_TITLEBAR:
-    case META_PREF_AUTO_RAISE:
-    case META_PREF_AUTO_RAISE_DELAY:
-    case META_PREF_TITLEBAR_FONT:
-    case META_PREF_NUM_WORKSPACES:
-    case META_PREF_KEYBINDINGS:
-    case META_PREF_DISABLE_WORKAROUNDS:
-    case META_PREF_BUTTON_LAYOUT:
-    case META_PREF_WORKSPACE_NAMES:
-    case META_PREF_VISUAL_BELL:
-    case META_PREF_AUDIBLE_BELL:
-    case META_PREF_VISUAL_BELL_TYPE:
-    case META_PREF_REDUCED_RESOURCES:
-    case META_PREF_GNOME_ACCESSIBILITY:
-    case META_PREF_GNOME_ANIMATIONS:
-    case META_PREF_RESIZE_WITH_RIGHT_BUTTON:
-    case META_PREF_EDGE_TILING:
-    case META_PREF_FORCE_FULLSCREEN:
-    case META_PREF_PLACEMENT_MODE:
-    case META_PREF_ALT_TAB_THUMBNAILS:
-      break;
-
-    default:
-      /* handled elsewhere or otherwise */
-      break;
-    }
+  meta_quit ();
 }
