@@ -242,13 +242,13 @@ reload_xinerama_infos (MetaScreen *screen)
       n_monitors = 0;
       result = XineramaGetInfo (screen->display->xdisplay,
                                 screen->number,
-				monitors, hints,
+                                monitors, hints,
                                 &n_monitors);
       /* Yes I know it should be Success but the current implementation
        * returns the num of monitor
        */
       if (result > 0)
-	{
+        {
           g_assert (n_monitors > 0);
 
           screen->xinerama_infos = g_new (MetaXineramaScreenInfo, n_monitors);
@@ -273,7 +273,7 @@ reload_xinerama_infos (MetaScreen *screen)
 
               ++i;
             }
-	}
+        }
     }
   else if (screen->n_xinerama_infos == 0)
     {
@@ -342,6 +342,7 @@ meta_screen_new (MetaDisplay *display,
   char buf[128];
   guint32 manager_timestamp;
   gulong current_workspace;
+  gboolean composited;
 
   replace_current_wm = meta_get_replace_current_wm ();
 
@@ -362,8 +363,7 @@ meta_screen_new (MetaDisplay *display,
    */
   if (xroot == None)
     {
-      meta_warning (_("Screen %d on display '%s' is invalid\n"),
-                    number, display->name);
+      g_warning ("Screen %d on display '%s' is invalid", number, display->name);
       return NULL;
     }
 
@@ -378,8 +378,9 @@ meta_screen_new (MetaDisplay *display,
 
       if (!replace_current_wm)
         {
-          meta_warning (_("Screen %d on display \"%s\" already has a window manager; try using the --replace option to replace the current window manager.\n"),
-                        number, display->name);
+          g_warning ("Screen %d on display \"%s\" already has a window "
+                     "manager; try using the --replace option to replace the "
+                     "current window manager.", number, display->name);
 
           return NULL;
         }
@@ -405,8 +406,8 @@ meta_screen_new (MetaDisplay *display,
 
   if (XGetSelectionOwner (xdisplay, wm_sn_atom) != new_wm_sn_owner)
     {
-      meta_warning (_("Could not acquire window manager selection on screen %d display \"%s\"\n"),
-                    number, display->name);
+      g_warning ("Could not acquire window manager selection on "
+                 "screen %d display \"%s\"", number, display->name);
 
       XDestroyWindow (xdisplay, new_wm_sn_owner);
 
@@ -461,8 +462,8 @@ meta_screen_new (MetaDisplay *display,
                 attr.your_event_mask);
   if (meta_error_trap_pop_with_return (display) != Success)
     {
-      meta_warning (_("Screen %d on display \"%s\" already has a window manager\n"),
-                    number, display->name);
+      g_warning ("Screen %d on display \"%s\" already has a window manager",
+                 number, display->name);
 
       XDestroyWindow (xdisplay, new_wm_sn_owner);
 
@@ -489,10 +490,6 @@ meta_screen_new (MetaDisplay *display,
   screen->wm_sn_atom = wm_sn_atom;
   screen->wm_sn_timestamp = manager_timestamp;
 
-  screen->wm_cm_selection_window = meta_create_offscreen_window (xdisplay,
-                                                                 xroot,
-                                                                 NoEventMask);
-
   screen->work_area_idle = 0;
 
   screen->active_workspace = NULL;
@@ -501,7 +498,6 @@ meta_screen_new (MetaDisplay *display,
   screen->columns_of_workspaces = -1;
   screen->vertical_workspaces = FALSE;
   screen->starting_corner = META_SCREEN_TOPLEFT;
-  screen->compositor_data = NULL;
 
   {
     XFontStruct *font_info;
@@ -524,7 +520,9 @@ meta_screen_new (MetaDisplay *display,
         XFreeFontInfo (NULL, font_info, 1);
       }
     else
-      meta_warning ("xserver doesn't have 'fixed' font.\n");
+      {
+        g_warning ("xserver doesn't have 'fixed' font.");
+      }
 
     screen->root_xor_gc = XCreateGC (screen->display->xdisplay,
                                      screen->xroot,
@@ -583,8 +581,8 @@ meta_screen_new (MetaDisplay *display,
   screen->keys_grabbed = FALSE;
   meta_screen_grab_keys (screen);
 
-  screen->ui = meta_ui_new (screen->display->xdisplay,
-                            screen->xscreen);
+  composited = meta_compositor_is_composited (screen->display->compositor);
+  screen->ui = meta_ui_new (screen->display->xdisplay, composited);
 
   screen->tab_popup = NULL;
   screen->tile_preview = NULL;
@@ -592,6 +590,7 @@ meta_screen_new (MetaDisplay *display,
   screen->tile_preview_timeout_id = 0;
 
   screen->stack = meta_stack_new (screen);
+  screen->stack_tracker = meta_stack_tracker_new (screen);
 
   meta_prefs_add_listener (prefs_changed_callback, screen);
 
@@ -627,19 +626,10 @@ void
 meta_screen_free (MetaScreen *screen,
                   guint32     timestamp)
 {
-  MetaDisplay *display;
   XGCValues gc_values = { 0 };
 
-  display = screen->display;
-
   screen->closing += 1;
 
-  meta_display_grab (display);
-
-  meta_compositor_unmanage_screen (display->compositor, screen);
-
-  meta_display_unmanage_windows_for_screen (display, screen, timestamp);
-
   meta_prefs_remove_listener (prefs_changed_callback, screen);
 
   meta_screen_ungrab_keys (screen);
@@ -665,12 +655,15 @@ meta_screen_free (MetaScreen *screen,
   meta_ui_free (screen->ui);
 
   meta_stack_free (screen->stack);
+  meta_stack_tracker_free (screen->stack_tracker);
 
   meta_error_trap_push (screen->display);
   XSelectInput (screen->display->xdisplay, screen->xroot, 0);
   if (meta_error_trap_pop_with_return (screen->display) != Success)
-    meta_warning (_("Could not release screen %d on display \"%s\"\n"),
-                  screen->number, screen->display->name);
+    {
+      g_warning ("Could not release screen %d on display \"%s\"",
+                 screen->number, screen->display->name);
+    }
 
   unset_wm_check_hint (screen);
 
@@ -704,130 +697,55 @@ meta_screen_free (MetaScreen *screen,
 
   g_free (screen->screen_name);
   g_free (screen);
-
-  XFlush (display->xdisplay);
-  meta_display_ungrab (display);
-}
-
-typedef struct
-{
-  Window		xwindow;
-  XWindowAttributes	attrs;
-} WindowInfo;
-
-static GList *
-list_windows (MetaScreen *screen)
-{
-  Window ignored1, ignored2;
-  Window *children;
-  guint n_children, i;
-  GList *result;
-
-  XQueryTree (screen->display->xdisplay,
-              screen->xroot,
-              &ignored1, &ignored2, &children, &n_children);
-
-  result = NULL;
-  for (i = 0; i < n_children; ++i)
-    {
-      WindowInfo *info = g_new0 (WindowInfo, 1);
-
-      meta_error_trap_push (screen->display);
-
-      XGetWindowAttributes (screen->display->xdisplay,
-                            children[i], &info->attrs);
-
-      if (meta_error_trap_pop_with_return (screen->display))
-	{
-          meta_verbose ("Failed to get attributes for window 0x%lx\n",
-                        children[i]);
-	  g_free (info);
-        }
-      else
-        {
-	  info->xwindow = children[i];
-	}
-
-      result = g_list_prepend (result, info);
-    }
-
-  if (children)
-    XFree (children);
-
-  return g_list_reverse (result);
 }
 
 void
 meta_screen_manage_all_windows (MetaScreen *screen)
 {
-  GList *windows;
-  GList *list;
-
-  meta_display_grab (screen->display);
-
-  windows = list_windows (screen);
+  Window *windows;
+  int n_windows;
+  Window *xwindows;
+  int i;
 
   meta_stack_freeze (screen->stack);
-  for (list = windows; list != NULL; list = list->next)
+  meta_stack_tracker_get_stack (screen->stack_tracker, &windows, &n_windows);
+
+  /* Copy the stack as it will be modified as part of the loop */
+  xwindows = g_memdup (windows, sizeof (Window) * n_windows);
+
+  for (i = 0; i < n_windows; i++)
     {
-      WindowInfo *info = list->data;
-      MetaWindow *window;
-
-      window = meta_window_new_with_attrs (screen->display, info->xwindow, TRUE,
-                                           &info->attrs);
-      if (info->xwindow == screen->no_focus_window ||
-          info->xwindow == screen->flash_window ||
-          info->xwindow == screen->wm_cm_selection_window ||
-          info->xwindow == screen->wm_sn_selection_window) {
-        meta_verbose ("Not managing our own windows\n");
-        continue;
-      }
-
-      meta_compositor_add_window (screen->display->compositor, window,
-                                  info->xwindow, &info->attrs);
+      meta_window_new (screen->display, xwindows[i], TRUE,
+                       META_EFFECT_TYPE_NONE);
     }
+
+  g_free (xwindows);
+
   meta_stack_thaw (screen->stack);
-
-  g_list_foreach (windows, (GFunc)g_free, NULL);
-  g_list_free (windows);
-
-  meta_display_ungrab (screen->display);
 }
 
 void
 meta_screen_composite_all_windows (MetaScreen *screen)
 {
   MetaDisplay *display;
-  GList *windows, *list;
+  GSList *windows, *list;
 
   display = screen->display;
-
-  windows = list_windows (screen);
-
-  meta_stack_freeze (screen->stack);
+  windows = meta_display_list_windows (display, META_LIST_INCLUDE_OVERRIDE_REDIRECT);
 
   for (list = windows; list != NULL; list = list->next)
     {
-      WindowInfo *info = list->data;
       MetaWindow *window;
 
-      if (info->xwindow == screen->no_focus_window ||
-          info->xwindow == screen->flash_window ||
-          info->xwindow == screen->wm_sn_selection_window ||
-          info->xwindow == screen->wm_cm_selection_window) {
-        meta_verbose ("Not managing our own windows\n");
-        continue;
-      }
+      window = list->data;
 
-      window = meta_display_lookup_x_window (display, info->xwindow);
-      meta_compositor_add_window (display->compositor, window,
-                                  info->xwindow, &info->attrs);
+      meta_compositor_add_window (display->compositor, window);
     }
 
-  meta_stack_thaw (screen->stack);
+  g_slist_free (windows);
 
-  g_list_foreach (windows, (GFunc)g_free, NULL);
-  g_list_free (windows);
+  /* initialize the compositor's view of the stacking order */
+  meta_stack_tracker_sync_stack (screen->stack_tracker);
 }
 
 MetaScreen*
@@ -950,7 +868,7 @@ meta_screen_foreach_window (MetaScreen *screen,
         {
           MetaWindow *window = tmp->data;
 
-          if (window->screen == screen)
+          if (window->screen == screen && !window->override_redirect)
             (* func) (screen, window, data);
         }
 
@@ -1019,7 +937,7 @@ meta_screen_get_workspace_by_index (MetaScreen  *screen,
 
 static void
 set_number_of_spaces_hint (MetaScreen *screen,
-			   int         n_spaces)
+                           int         n_spaces)
 {
   unsigned long data[1];
 
@@ -1194,7 +1112,7 @@ meta_screen_update_cursor (MetaScreen *screen)
   Cursor xcursor;
 
   xcursor = meta_display_create_x_cursor (screen->display,
-					  screen->current_cursor);
+                                          screen->current_cursor);
   XDefineCursor (screen->display->xdisplay, screen->xroot, xcursor);
   XFlush (screen->display->xdisplay);
   XFreeCursor (screen->display->xdisplay, xcursor);
@@ -1564,7 +1482,7 @@ meta_screen_get_mouse_window (MetaScreen  *screen,
 
 const MetaXineramaScreenInfo*
 meta_screen_get_xinerama_for_rect (MetaScreen    *screen,
-				   MetaRectangle *rect)
+                                   MetaRectangle *rect)
 {
   int i;
   int best_xinerama, xinerama_score;
@@ -1596,7 +1514,7 @@ meta_screen_get_xinerama_for_rect (MetaScreen    *screen,
 
 const MetaXineramaScreenInfo*
 meta_screen_get_xinerama_for_window (MetaScreen *screen,
-				     MetaWindow *window)
+                                     MetaWindow *window)
 {
   MetaRectangle window_rect;
 
@@ -1817,7 +1735,7 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
               screen->vertical_workspaces = TRUE;
               break;
             default:
-              meta_warning ("Someone set a weird orientation in _NET_DESKTOP_LAYOUT\n");
+              g_warning ("Someone set a weird orientation in _NET_DESKTOP_LAYOUT");
               break;
             }
 
@@ -1826,7 +1744,8 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
 
           if (rows <= 0 && cols <= 0)
             {
-              meta_warning ("Columns = %d rows = %d in _NET_DESKTOP_LAYOUT makes no sense\n", rows, cols);
+              g_warning ("Columns = %d rows = %d in _NET_DESKTOP_LAYOUT makes no sense",
+                         rows, cols);
             }
           else
             {
@@ -1858,7 +1777,7 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
                     screen->starting_corner = META_SCREEN_BOTTOMLEFT;
                     break;
                   default:
-                    meta_warning ("Someone set a weird starting corner in _NET_DESKTOP_LAYOUT\n");
+                    g_warning ("Someone set a weird starting corner in _NET_DESKTOP_LAYOUT");
                     break;
                 }
             }
@@ -1867,8 +1786,8 @@ meta_screen_update_workspace_layout (MetaScreen *screen)
         }
       else
         {
-          meta_warning ("Someone set _NET_DESKTOP_LAYOUT to %d integers instead of 4 "
-                        "(3 is accepted for backwards compat)\n", n_items);
+          g_warning ("Someone set _NET_DESKTOP_LAYOUT to %d integers instead of 4 "
+                     "(3 is accepted for backwards compat)", n_items);
         }
 
       meta_XFree (list);
@@ -1915,9 +1834,9 @@ set_workspace_names (MetaScreen *screen)
   XChangeProperty (screen->display->xdisplay,
                    screen->xroot,
                    screen->display->atom__NET_DESKTOP_NAMES,
-		   screen->display->atom_UTF8_STRING,
+                   screen->display->atom_UTF8_STRING,
                    8, PropModeReplace,
-		   (unsigned char *)flattened->str, flattened->len);
+                   (unsigned char *)flattened->str, flattened->len);
   meta_error_trap_pop (screen->display);
 
   g_string_free (flattened, TRUE);
@@ -2010,7 +1929,7 @@ set_work_area_hint (MetaScreen *screen)
           tmp[2] = area.width;
           tmp[3] = area.height;
 
-	  tmp += 4;
+          tmp += 4;
         }
 
       tmp_list = tmp_list->next;
@@ -2018,9 +1937,9 @@ set_work_area_hint (MetaScreen *screen)
 
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
-		   screen->display->atom__NET_WORKAREA,
-		   XA_CARDINAL, 32, PropModeReplace,
-		   (guchar*) data, num_workspaces*4);
+                   screen->display->atom__NET_WORKAREA,
+                   XA_CARDINAL, 32, PropModeReplace,
+                   (guchar*) data, num_workspaces*4);
   g_free (data);
   meta_error_trap_pop (screen->display);
 }
@@ -2047,15 +1966,13 @@ meta_screen_queue_workarea_recalc (MetaScreen *screen)
       meta_topic (META_DEBUG_WORKAREA,
                   "Adding work area hint idle function\n");
       screen->work_area_idle =
-        g_idle_add_full (META_PRIORITY_WORK_AREA_HINT,
+        g_idle_add_full (META_PRIORITY_BEFORE_REDRAW,
                          (GSourceFunc) set_work_area_idle_func,
                          screen,
                          NULL);
     }
 }
 
-
-#ifdef WITH_VERBOSE_MODE
 static const gchar *
 meta_screen_corner_to_string (MetaScreenCorner corner)
 {
@@ -2075,7 +1992,6 @@ meta_screen_corner_to_string (MetaScreenCorner corner)
 
   return "Unknown";
 }
-#endif /* WITH_VERBOSE_MODE */
 
 void
 meta_screen_calc_workspace_layout (MetaScreen          *screen,
@@ -2289,8 +2205,8 @@ meta_screen_calc_workspace_layout (MetaScreen          *screen,
     }
 
   if (i != grid_area)
-    meta_bug ("did not fill in the whole workspace grid in %s (%d filled)\n",
-              G_STRFUNC, i);
+    g_error ("did not fill in the whole workspace grid in %s (%d filled)",
+             G_STRFUNC, i);
 
   current_row = 0;
   current_col = 0;
@@ -2322,8 +2238,7 @@ meta_screen_calc_workspace_layout (MetaScreen          *screen,
   layout->current_row = current_row;
   layout->current_col = current_col;
 
-#ifdef WITH_VERBOSE_MODE
-  if (meta_is_verbose ())
+  if (meta_check_debug_flags (META_DEBUG_VERBOSE))
     {
       r = 0;
       while (r < layout->rows)
@@ -2345,7 +2260,6 @@ meta_screen_calc_workspace_layout (MetaScreen          *screen,
           ++r;
         }
     }
-#endif /* WITH_VERBOSE_MODE */
 }
 
 void
@@ -2379,6 +2293,8 @@ meta_screen_resize (MetaScreen *screen,
   reload_xinerama_infos (screen);
   set_desktop_geometry_hint (screen);
 
+  meta_compositor_sync_screen_size (screen->display->compositor);
+
   /* Queue a resize on all the windows */
   meta_screen_foreach_window (screen, meta_screen_resize_func, 0);
 }
@@ -2408,7 +2324,7 @@ queue_windows_showing (MetaScreen *screen)
    * active_workspace's window list, because the active_workspace's
    * window list may not contain the on_all_workspace windows.
    */
-  windows = meta_display_list_windows (screen->display);
+  windows = meta_display_list_windows (screen->display, META_LIST_DEFAULT);
 
   tmp = windows;
   while (tmp != NULL)
@@ -2438,10 +2354,8 @@ meta_screen_minimize_all_on_active_workspace_except (MetaScreen *screen,
     {
       MetaWindow *w = tmp->data;
 
-      if (w->screen == screen  &&
-          w->has_minimize_func &&
-	  w != keep)
-	meta_window_minimize (w);
+      if (w->screen == screen && w->has_minimize_func && w != keep)
+        meta_window_minimize (w);
 
       tmp = tmp->next;
     }
@@ -2859,44 +2773,3 @@ meta_screen_get_size (MetaScreen *screen,
   *width = screen->rect.width;
   *height = screen->rect.height;
 }
-
-gpointer
-meta_screen_get_compositor_data (MetaScreen *screen)
-{
-  return screen->compositor_data;
-}
-
-void
-meta_screen_set_compositor_data (MetaScreen *screen,
-                                 gpointer    compositor)
-{
-  screen->compositor_data = compositor;
-}
-
-void
-meta_screen_set_cm_selection (MetaScreen *screen)
-{
-  char selection[32];
-  Atom a;
-
-  screen->wm_cm_timestamp = meta_display_get_current_time_roundtrip (
-                                                               screen->display);
-
-  g_snprintf (selection, sizeof(selection), "_NET_WM_CM_S%d", screen->number);
-  meta_verbose ("Setting selection: %s\n", selection);
-  a = XInternAtom (screen->display->xdisplay, selection, FALSE);
-  XSetSelectionOwner (screen->display->xdisplay, a,
-                      screen->wm_cm_selection_window, screen->wm_cm_timestamp);
-}
-
-void
-meta_screen_unset_cm_selection (MetaScreen *screen)
-{
-  char selection[32];
-  Atom a;
-
-  g_snprintf (selection, sizeof(selection), "_NET_WM_CM_S%d", screen->number);
-  a = XInternAtom (screen->display->xdisplay, selection, FALSE);
-  XSetSelectionOwner (screen->display->xdisplay, a,
-                      None, screen->wm_cm_timestamp);
-}
