@@ -34,7 +34,7 @@
 
 void meta_workspace_queue_calc_showing   (MetaWorkspace *workspace);
 static void set_active_space_hint        (MetaScreen *screen);
-static void focus_ancestor_or_mru_window (MetaWorkspace *workspace,
+static void focus_ancestor_or_top_window (MetaWorkspace *workspace,
                                           MetaWindow    *not_this_one,
                                           guint32        timestamp);
 static void free_this                    (gpointer candidate,
@@ -328,7 +328,7 @@ static void workspace_switch_sound(MetaWorkspace *from,
       break;
 
   if (i >= nw) {
-    meta_bug("Failed to find destination workspace in layout\n");
+    g_error ("Failed to find destination workspace in layout");
     goto finish;
   }
 
@@ -343,7 +343,7 @@ static void workspace_switch_sound(MetaWorkspace *from,
      movement but not such much vertical movement. */
 
   if (x == layout.current_col && y == layout.current_row) {
-    meta_bug("Uh, origin and destination workspace at same logic position!\n");
+    g_error ("Uh, origin and destination workspace at same logic position!");
     goto finish;
   }
 
@@ -469,7 +469,7 @@ meta_workspace_index (MetaWorkspace *workspace)
   ret = g_list_index (workspace->screen->workspaces, workspace);
 
   if (ret < 0)
-    meta_bug ("Workspace does not exist to index!\n");
+    g_error ("Workspace does not exist to index!");
 
   return ret;
 }
@@ -484,7 +484,8 @@ meta_workspace_list_windows (MetaWorkspace *workspace)
   GSList *tmp;
   GList *workspace_windows;
 
-  display_windows = meta_display_list_windows (workspace->screen->display);
+  display_windows = meta_display_list_windows (workspace->screen->display,
+                                               META_LIST_DEFAULT);
 
   workspace_windows = NULL;
   tmp = display_windows;
@@ -527,7 +528,7 @@ set_active_space_hint (MetaScreen *screen)
                    screen->display->atom__NET_CURRENT_DESKTOP,
                    XA_CARDINAL,
                    32, PropModeReplace, (guchar*) data, 1);
-  meta_error_trap_pop (screen->display, FALSE);
+  meta_error_trap_pop (screen->display);
 }
 
 void
@@ -652,9 +653,10 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
 #define MIN_SANE_AREA 100
   if (work_area.width < MIN_SANE_AREA)
     {
-      meta_warning ("struts occupy an unusually large percentage of the screen; "
-                    "available remaining width = %d < %d",
-                    work_area.width, MIN_SANE_AREA);
+      g_warning ("struts occupy an unusually large percentage of the screen; "
+                 "available remaining width = %d < %d",
+                 work_area.width, MIN_SANE_AREA);
+
       if (work_area.width < 1)
         {
           work_area.x = (workspace->screen->rect.width - MIN_SANE_AREA)/2;
@@ -669,9 +671,10 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
     }
   if (work_area.height < MIN_SANE_AREA)
     {
-      meta_warning ("struts occupy an unusually large percentage of the screen; "
-                    "available remaining height = %d < %d",
-                    work_area.height, MIN_SANE_AREA);
+      g_warning ("struts occupy an unusually large percentage of the screen; "
+                 "available remaining height = %d < %d",
+                  work_area.height, MIN_SANE_AREA);
+
       if (work_area.height < 1)
         {
           work_area.y = (workspace->screen->rect.height - MIN_SANE_AREA)/2;
@@ -791,7 +794,6 @@ meta_workspace_get_onxinerama_region (MetaWorkspace *workspace,
   return workspace->xinerama_region[which_xinerama];
 }
 
-#ifdef WITH_VERBOSE_MODE
 static const gchar *
 meta_motion_direction_to_string (MetaMotionDirection direction)
 {
@@ -811,7 +813,6 @@ meta_motion_direction_to_string (MetaMotionDirection direction)
 
   return "Unknown";
 }
-#endif /* WITH_VERBOSE_MODE */
 
 MetaWorkspace*
 meta_workspace_get_neighbor (MetaWorkspace      *workspace,
@@ -864,8 +865,10 @@ meta_workspace_get_neighbor (MetaWorkspace      *workspace,
     i = current_space;
 
   if (i >= num_workspaces)
-    meta_bug ("calc_workspace_layout left an invalid (too-high) workspace number %d in the grid\n",
-              i);
+    {
+      g_error ("calc_workspace_layout left an invalid (too-high) workspace "
+               "number %d in the grid", i);
+    }
 
   meta_verbose ("Neighbor workspace is %d at row %d col %d\n",
                 i, layout.current_row, layout.current_col);
@@ -888,14 +891,13 @@ meta_workspace_focus_default_window (MetaWorkspace *workspace,
 {
   if (timestamp == CurrentTime)
     {
-      meta_warning ("CurrentTime used to choose focus window; "
-                    "focus window may not be correct.\n");
+      g_warning ("CurrentTime used to choose focus window; "
+                 "focus window may not be correct.");
     }
 
-
   if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_CLICK ||
       !workspace->screen->display->mouse_mode)
-    focus_ancestor_or_mru_window (workspace, not_this_one, timestamp);
+    focus_ancestor_or_top_window (workspace, not_this_one, timestamp);
   else
     {
       MetaWindow * window;
@@ -932,7 +934,7 @@ meta_workspace_focus_default_window (MetaWorkspace *workspace,
             }
         }
       else if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_SLOPPY)
-        focus_ancestor_or_mru_window (workspace, not_this_one, timestamp);
+        focus_ancestor_or_top_window (workspace, not_this_one, timestamp);
       else if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_MOUSE)
         {
           meta_topic (META_DEBUG_FOCUS,
@@ -955,17 +957,13 @@ record_ancestor (MetaWindow *window,
   return FALSE; /* quit with the first ancestor we find */
 }
 
-/* Focus ancestor of not_this_one if there is one, otherwise focus the MRU
- * window on active workspace
- */
+/* Focus ancestor of not_this_one if there is one */
 static void
-focus_ancestor_or_mru_window (MetaWorkspace *workspace,
+focus_ancestor_or_top_window (MetaWorkspace *workspace,
                               MetaWindow    *not_this_one,
                               guint32        timestamp)
 {
   MetaWindow *window = NULL;
-  MetaWindow *desktop_window = NULL;
-  GList *tmp;
 
   if (not_this_one)
     meta_topic (META_DEBUG_FOCUS,
@@ -996,36 +994,8 @@ focus_ancestor_or_mru_window (MetaWorkspace *workspace,
         }
     }
 
-  /* No ancestor, look for the MRU window */
-  tmp = workspace->mru_list;
-
-  while (tmp)
-    {
-      MetaWindow* tmp_window;
-      tmp_window = ((MetaWindow*) tmp->data);
-      if (tmp_window != not_this_one           &&
-          meta_window_showing_on_its_workspace (tmp_window) &&
-          tmp_window->type != META_WINDOW_DOCK &&
-          tmp_window->type != META_WINDOW_DESKTOP)
-        {
-          window = tmp->data;
-          break;
-        }
-      else if (tmp_window != not_this_one      &&
-               desktop_window == NULL          &&
-               meta_window_showing_on_its_workspace (tmp_window) &&
-               tmp_window->type == META_WINDOW_DESKTOP)
-        {
-          /* Found the most recently used desktop window */
-          desktop_window = tmp_window;
-        }
-
-      tmp = tmp->next;
-    }
-
-  /* If no window was found, default to the MRU desktop-window */
-  if (window == NULL)
-    window = desktop_window;
+  window = meta_stack_get_default_focus_window (workspace->screen->stack,
+                                                workspace, not_this_one);
 
   if (window)
     {
