@@ -34,20 +34,15 @@
 #include <cairo-xlib.h>
 #include <libmetacity/meta-theme.h>
 
-/**
- * The current theme. (Themes are singleton.)
- */
-static MetaTheme *meta_current_theme = NULL;
-
-static void meta_ui_accelerator_parse (const char      *accel,
-                                       guint           *keysym,
-                                       guint           *keycode,
-                                       GdkModifierType *keymask);
-
 struct _MetaUI
 {
   Display *xdisplay;
-  Screen *xscreen;
+
+  gboolean composited;
+  gint scale;
+  gdouble dpi;
+
+  MetaTheme *theme;
   MetaFrames *frames;
 
   /* For double-click tracking */
@@ -58,6 +53,70 @@ struct _MetaUI
   guint32 button_click_time;
 };
 
+static gboolean
+get_int_setting (const gchar *name,
+                 gint        *value)
+{
+  GValue gvalue = G_VALUE_INIT;
+
+  g_value_init (&gvalue, G_TYPE_INT);
+
+  if (gdk_screen_get_setting (gdk_screen_get_default (), name, &gvalue))
+    {
+      *value = g_value_get_int (&gvalue);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gint
+get_window_scaling_factor (MetaUI *ui)
+{
+  gint scale;
+
+  if (get_int_setting ("gdk-window-scaling-factor", &scale))
+    return scale;
+
+  return 1;
+}
+
+static gdouble
+get_xft_dpi (MetaUI *ui)
+{
+  const gchar *dpi_scale_env;
+  gint xft_dpi;
+  gdouble dpi;
+
+  dpi = 96.0;
+  if (get_int_setting ("gtk-xft-dpi", &xft_dpi) && xft_dpi > 0)
+    dpi = xft_dpi / 1024.0 / ui->scale;
+
+  dpi_scale_env = g_getenv ("GDK_DPI_SCALE");
+  if (dpi_scale_env != NULL)
+    {
+      gdouble dpi_scale;
+
+      dpi_scale = g_ascii_strtod (dpi_scale_env, NULL);
+      if (dpi_scale != 0)
+        dpi *= dpi_scale;
+    }
+
+  return dpi;
+}
+
+static void
+notify_gtk_xft_dpi_cb (GtkSettings *settings,
+                       GParamSpec  *pspec,
+                       MetaUI      *ui)
+{
+  ui->scale = get_window_scaling_factor (ui);
+  ui->dpi = get_xft_dpi (ui);
+
+  meta_theme_set_scale (ui->theme, ui->scale);
+  meta_theme_set_dpi (ui->theme, ui->dpi);
+}
+
 void
 meta_ui_init (int *argc, char ***argv)
 {
@@ -74,7 +133,10 @@ meta_ui_init (int *argc, char ***argv)
   gdk_set_allowed_backends ("x11");
 
   if (!gtk_init_check (argc, argv))
-    meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
+    {
+      g_critical ("Unable to open X display %s", XDisplayName (NULL));
+      exit (EXIT_FAILURE);
+    }
 
   /* We need to be able to fully trust that the window and monitor sizes
    * that GDK reports corresponds to the X ones, so we disable the automatic
@@ -277,21 +339,29 @@ meta_ui_remove_event_func (Display       *xdisplay,
 }
 
 MetaUI*
-meta_ui_new (Display *xdisplay,
-             Screen  *screen)
+meta_ui_new (Display  *xdisplay,
+             gboolean  composited)
 {
   GdkDisplay *gdisplay;
   MetaUI *ui;
 
   ui = g_new0 (MetaUI, 1);
   ui->xdisplay = xdisplay;
-  ui->xscreen = screen;
+
+  ui->composited = composited;
+  ui->scale = get_window_scaling_factor (ui);
+  ui->dpi = get_xft_dpi (ui);
+
+  g_signal_connect (gtk_settings_get_default (), "notify::gtk-xft-dpi",
+                    G_CALLBACK (notify_gtk_xft_dpi_cb), ui);
 
   gdisplay = gdk_x11_lookup_xdisplay (xdisplay);
   g_assert (gdisplay == gdk_display_get_default ());
-
   g_assert (xdisplay == GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-  ui->frames = meta_frames_new ();
+
+  meta_ui_reload_theme (ui);
+
+  ui->frames = meta_frames_new (ui);
 
   /* GTK+ needs the frame-sync protocol to work in order to properly
    * handle style changes. This means that the dummy widget we create
@@ -319,6 +389,25 @@ meta_ui_free (MetaUI *ui)
   g_free (ui);
 }
 
+void
+meta_ui_set_composited (MetaUI   *ui,
+                        gboolean  composited)
+{
+  if (ui->composited == composited)
+    return;
+
+  ui->composited = composited;
+
+  meta_theme_set_composited (ui->theme, composited);
+  meta_frames_composited_changed (ui->frames);
+}
+
+gint
+meta_ui_get_scale (MetaUI *ui)
+{
+  return ui->scale;
+}
+
 void
 meta_ui_get_frame_mask (MetaUI  *ui,
                         Window   frame_xwindow,
@@ -355,7 +444,8 @@ meta_ui_create_frame_window (MetaUI  *ui,
                              gint     x,
                              gint     y,
                              gint     width,
-                             gint     height)
+                             gint     height,
+                             gulong  *create_serial)
 {
   GdkScreen *screen = gdk_screen_get_default ();
   GdkWindowAttr attrs;
@@ -400,6 +490,12 @@ meta_ui_create_frame_window (MetaUI  *ui,
 
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
+  /* We make an assumption that gdk_window_new() is going to call
+   * XCreateWindow as it's first operation; this seems to be true currently
+   * as long as you pass in a colormap.
+   */
+  if (create_serial)
+    *create_serial = XNextRequest (xdisplay);
   window =
     gdk_window_new (gdk_screen_get_root_window(screen),
 		    &attrs, attributes_mask);
@@ -680,16 +776,22 @@ meta_ui_theme_get_frame_borders (MetaUI           *ui,
   MetaTheme *theme;
   const gchar *theme_variant;
 
-  theme = meta_ui_get_theme ();
+  theme = meta_ui_get_theme (ui);
   theme_variant = NULL;
 
   meta_theme_get_frame_borders (theme, theme_variant, type, flags, borders);
 }
 
 MetaTheme *
-meta_ui_get_theme (void)
+meta_ui_get_theme (MetaUI *ui)
 {
-  return meta_current_theme;
+  return ui->theme;
+}
+
+gboolean
+meta_ui_is_composited (MetaUI *ui)
+{
+  return ui->composited;
 }
 
 static gchar *
@@ -714,11 +816,11 @@ get_theme_name (MetaThemeType theme_type)
 }
 
 static MetaTheme *
-load_theme (MetaThemeType  theme_type,
+load_theme (MetaUI        *ui,
+            MetaThemeType  theme_type,
             const gchar   *theme_name)
 {
   MetaTheme *theme;
-  gboolean compositing_manager;
   const PangoFontDescription *titlebar_font;
   GError *error;
   const gchar *button_layout;
@@ -726,8 +828,9 @@ load_theme (MetaThemeType  theme_type,
 
   theme = meta_theme_new (theme_type);
 
-  compositing_manager = meta_prefs_get_compositing_manager ();
-  meta_theme_set_composited (theme, compositing_manager);
+  meta_theme_set_composited (theme, ui->composited);
+  meta_theme_set_scale (theme, ui->scale);
+  meta_theme_set_dpi (theme, ui->dpi);
 
   titlebar_font = meta_prefs_get_titlebar_font ();
   meta_theme_set_titlebar_font (theme, titlebar_font);
@@ -751,7 +854,7 @@ load_theme (MetaThemeType  theme_type,
 }
 
 void
-meta_ui_reload_theme (void)
+meta_ui_reload_theme (MetaUI *ui)
 {
   MetaThemeType theme_type;
   gchar *theme_name;
@@ -760,23 +863,23 @@ meta_ui_reload_theme (void)
   theme_type = meta_prefs_get_theme_type ();
   theme_name = get_theme_name (theme_type);
 
-  theme = load_theme (theme_type, theme_name);
+  theme = load_theme (ui, theme_type, theme_name);
   g_free (theme_name);
 
   if (theme == NULL)
     {
       g_warning (_("Falling back to default GTK+ theme - Adwaita"));
-      theme = load_theme (META_THEME_TYPE_GTK, "Adwaita");
+      theme = load_theme (ui, META_THEME_TYPE_GTK, "Adwaita");
     }
 
-  g_set_object (&meta_current_theme, theme);
-  g_assert (meta_current_theme);
+  g_set_object (&ui->theme, theme);
+  g_assert (ui->theme);
 
   meta_invalidate_default_icons ();
 }
 
 void
-meta_ui_update_button_layout (void)
+meta_ui_update_button_layout (MetaUI *ui)
 {
   const gchar *button_layout;
   gboolean invert;
@@ -784,7 +887,7 @@ meta_ui_update_button_layout (void)
   button_layout = meta_prefs_get_button_layout ();
   invert = meta_ui_get_direction() == META_UI_DIRECTION_RTL;
 
-  meta_theme_set_button_layout (meta_current_theme, button_layout, invert);
+  meta_theme_set_button_layout (ui->theme, button_layout, invert);
 }
 
 static void
